// RB-100: Controlled KONDO KRS-2552 Servos by COM3
//

#include <stdio.h>
#include <stdlib.h>
#include "roboard.h"

#ifdef WINCE
	#include <windows.h>
#endif

#define ID				0

// KRS-2552 commands
#define POS_COMD		0x80
#define SPEED_COMD		0xc0

#ifdef WINCE
int _tmain(int argc, _TCHAR* _argv[])
{

#else
int main()
{
#endif

	unsigned char tx_buf[128];
	unsigned char rx_buf[128];
	unsigned int pos, i;

	// first set the correct RoBoard version
    roboio_SetRBVer(RB_100);		// if your RoBoard is the newest RB-100 (model: DM-118D)
    //roboio_SetRBVer(RB_100b2);	// if your RoBoard is old RB-100 (model: DM-118A/DM-118B/DM-118C)
    //roboio_SetRBVer(RB_100RD);	// if your RoBoard is RB-100RD
    //roboio_SetRBVer(RB_110);		// if your RoBoard is RB-110
    //roboio_SetRBVer(RB_050);		// if your RoBoard is RB-050

	if( !com3_Init(COM_HDUPLEX) )
    {
        printf("ERROR: %s\n", roboio_GetErrMsg()); return -1;
    }

	// Set data format as 8 bits, 1 stop bit, even parity
	com3_SetFormat(COM_BYTESIZE8,COM_STOPBIT1,COM_EVENPARITY);

	// Set COM3 baudrate to 115200bps (note: you should ensure that the baudrate of your servo is also set as 115200bps)
	com3_SetBaud(COMBAUD_115200BPS);


	// Set servo speed
    // TX: CMD SC SPD
	tx_buf[0] = SPEED_COMD + ID;				// CMD
	tx_buf[1] = 2;								// SC: Please reference the "ICS3.0 Comand Reference"
	tx_buf[2] = 100;							// SPD: Set speed to 100

	if( com3_ServoTRX(tx_buf,3,rx_buf,3) )
		printf("Set Speed to %u.\n",(unsigned int)rx_buf[2]);
	else
		printf("Failed to set Speed: %s\n",roboio_GetErrMsg());


	// Send position 3600 to servo (moving servo)
	// TX: CMD POS_H POS_L
	// POS_H(pos & 11111110000000b): high byte of position (7 bits)
	// POS_L(pos & 00000001111111b): low byte of position  (7 bits)
	tx_buf[0] = POS_COMD + ID;					// CMD
	tx_buf[1] = (3600 & 0x3f80)>>7;				// 0x3f80 = 11111110000000b
	tx_buf[2] = 3600 & 0x007f;					// 0x007f = 00000001111111b

	if( com3_ServoTRX(tx_buf,3,rx_buf,3) )
	{
		pos = (unsigned int)rx_buf[1] + ((unsigned int)rx_buf[2]<<7); // get the old position
		printf("The old position: %u\n",pos);
		printf("Moving servo (press enter to continue)...\n"); getchar();
	}
	else
		printf("Moving servo: failed to write position: %s\n",roboio_GetErrMsg());


	// Send position 10500 to servo (moving servo)
	// TX: CMD POS_H POS_L
	// POS_H(pos & 11111110000000b): high byte of position (7 bits)
	// POS_L(pos & 00000001111111b): low byte of position  (7 bits)
	tx_buf[0] = POS_COMD + ID;					// CMD
	tx_buf[1] = (10500 & 0x3f80)>>7;			// 0x3f80 = 11111110000000b
	tx_buf[2] = 10500 & 0x007f;					// 0x007f = 00000001111111b
	
	if( com3_ServoTRX(tx_buf,3,rx_buf,3) )
	{
		pos = (unsigned int)rx_buf[1] + ((unsigned int)rx_buf[2]<<7); // get the old position
		printf("The old position: %u\n",pos);
		printf("Moving servo again (press enter to continue)...\n"); getchar();
	}
	else
		printf("Moving servo again: failed to write position: %s\n",roboio_GetErrMsg());

	com3_Close();
	printf("\nDone.\n");

	return 0;
}
